/* Definitions of the set of intrinsic functions in Juno */
use phf::phf_map;

use crate::types::{Type, TypeSolver};
use crate::parser;

// How intrinsics are identified in the Hercules IR
pub type IntrinsicIdentity = usize;

// Information about a single intrinsic, including its type information and how it will be
// identified in the Hercules IR
#[derive(Clone)]
pub struct IntrinsicInfo {
    pub id      : IntrinsicIdentity,
    pub kinds   : &'static [parser::Kind],
    pub args    : fn(&Vec<Type>, &mut TypeSolver) -> Vec<(Type, bool)>,
    pub ret_typ : fn(&Vec<Type>, &mut TypeSolver) -> Type,
}

fn sqrt_args(ty_args : &Vec<Type>, _ : &mut TypeSolver) -> Vec<(Type, bool)> {
    vec![(ty_args[0], false)]
}

fn sqrt_return(ty_args : &Vec<Type>, _ : &mut TypeSolver) -> Type {
    ty_args[0]
}

static INTRINSICS : phf::Map<&'static str, IntrinsicInfo> = phf_map! {
    "sqrt" => IntrinsicInfo {
        id      : 0,
        kinds   : &[parser::Kind::Number],
        args    : sqrt_args,
        ret_typ : sqrt_return,
    },
};

pub fn lookup(nm : &str) -> Option<&IntrinsicInfo> {
    INTRINSICS.get(nm)
}
